<?php
/**
 * @file
 * The skeleton operation class.
 *
 * Note that the FILENAME of the operation is used as the
 * operation id in the scripts, and does not need to be the
 * same as the operations classname.
 */

/**
 * Declare the ctools plugin.
 *
 * This should at least have a label and
 * handler['class'], there is also an optional flag used by
 * restore generate ui 'uses_ids', if this is set to TRUE then a
 * warning will be displayed stating that the operation uses IDs
 * such as NIDs or UIDs.
 */
$plugin = array(
  // The label of the plugin, this is used in all the UI displays.
  'label' => t('Skeleton operation'),
  // TRUE if the operation uses IDs.
  'uses_ids' => TRUE,
  // The handlers class name.
  'handler' => array(
    'class' => 'RestoreOperationsSkeletonOperation',
  ),
);

/**
 * The operation class, must extend from RestoreOperation.
 */
class RestoreOperationsSkeletonOperation extends RestoreOperation {
  /**
   * @var array
   *  Static storage for the variables for generate.
   *
   * This is used in the available method.
   */
  protected static $variables;

  /**
   * Implements RestoreOperation->details().
   *
   * This method returns a label for a specific action for the
   * UI displays, for example "Content type: name".
   *
   * The $action given is the action from the script.
   */
  public function details(array $action) {
    // Return the name of the variable.
    return t('Skeleton operation: @name', array(
      '@name' => $action['name'],
    ));
  }

  /**
   * Implements RestoreOperation->status().
   *
   * This method returns the status of the action, if it
   * installed, uninstalled or overriden.
   *
   * Here you will usually attempt to load the data from the database
   * and compare it to the details in the $action from the
   * restore script.
   *
   * If its not available locally then this method should return
   * RESTORE_STATUS_UNINSTALLED, if the data is available but its
   * different to the restore script return RESTORE_STATUS_OVERRIDEN,
   * otherwise everything should match and RESTORE_STATUS_INSTALLED
   * should be returned.
   */
  public function status(array $action) {
    // Check if the variable exists.
    $exists = isset($GLOBALS['conf'][$action['name']]);

    // Check if the variable value matches the value stored in the
    // restore scripts action.
    $matches = $exists ? variable_get($action['name']) == $action['value'] : FALSE;

    // Returns:
    // RESTORE_STATUS_INSTALLED if $exists and $matches is TRUE,
    // RESTORE_STATUS_OVERRIDEN if $exists is TRUE but matches is FALSE,
    // RESTORE_STATUS_UNINSTALLED if $exists is FALSE.
    return $exists ? ($GLOBALS['conf'][$action['name']] == $action['value'] ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_OVERRIDEN) : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   *
   * This method does the restore work, and should update or insert the
   * data from the restore script.
   */
  public function restore(array $action) {
    // Set the variable to the value from the restore script.
    variable_set($action['name'], $action['value']);

    // Return TRUE if there was no problems, otherwise FALSE.
    return TRUE;
  }

  /**
   * Implements RestoreOperation->remove().
   *
   * This method removes the data from Drupal, for example by
   * deleting the variable, rule, etc.
   */
  public function remove(array $action) {
    // Delete the variable.
    variable_del($action['name']);

    // Return TRUE if there was no problems, otherwise FALSE.
    return TRUE;
  }

  /**
   * Implements RestoreOperation->conflict().
   *
   * This method checks if there are any other actions similar
   * to the one being checked.
   *
   * $action is the action being checked, and $actions is an array
   * of other actions found.
   *
   * Returns FALSE if there are no conflicts, otherwise an array
   * of details for each conflict.
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      // Check if $_action matches $action... if this case
      // we check if the variable names are the same.
      if (!($_action['name'] !== $action['name'])) {
        // They don't match so continue.
        continue;
      }

      // We found a conflicting action, so add the details to the
      // $restults array indexed by the $action_id.
      $results[$action_id] = $this->details($_action);
    }

    // If there are conflicts, return the array otherwise return FALSE.
    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   *
   * This method is for the diff UI, and is ment for building a nice
   * output for the display (e.g. may not want to display all the attributes
   * from a view or rule config).
   *
   * The logic is usually the same as the status method.
   */
  public function diff(array $action) {
    // Check if the variable exists.
    $exists = isset($GLOBALS['conf'][$action['name']]);

    // Check if the variable value matches the value stored in the
    // restore scripts action.
    $matches = $exists ? variable_get($action['name']) == $action['value'] : FALSE;

    // Return a diff data array.
    return array(
      // The value currently available to Drupal, this can be nicely formatted
      // for the UI.
      'current_value' => $exists ? variable_get($action['name']) : NULL,
      // The value stored in the restore script, and again can be nicely
      // formatted for the UI.
      'restore_value' => $action['value'],
      // If the current_value matches the restore_value, either TRUE or
      // FALSE.
      'match' => $exists && $matches,
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   *
   * This method is similar to the details method, except this gets the
   * details from Drupal and not the restore script, so this depends
   * if the data is available in Drupal.
   *
   * This is only used in the generate UI form when creating or altering
   * an existing script.
   *
   * The $id is provided from the generate form, which in turn is provided
   * by the available method.
   */
  public function detailsFromId($id) {
    // Check if the variable exists, if not return FALSE.
    if (!isset($GLOBALS['conf'][$action['name']])) {
      return FALSE;
    }

    // If the variable does exist, return the label.
    return t('Skeleton operation: @name', array(
      '@name' => $action['name'],
    ));
  }

  /**
   * Implements RestoreOperation->available().
   *
   * This method populates the generate form with available data
   * that can be placed in to the scripts, for example for variables
   * this method will return an array of variables indexed with the variable
   * name.
   *
   * The index of the returned array is used for other areas so must be unique.
   */
  public function available() {
    // Using a static to store the variables in so that we don't have to
    // keep rebuilding it if the operation is called multiple times.
    if (empty(static::$variables)) {
      // Use a db query to collect all the variable names.
      $dbq = db_select('variable', 'v')
        ->fields('v', array('name'))
        ->execute();

      // For each row, add it to the $variables static array, indexing it by
      // the variables name.
      while ($row = $dbq->fetchAssoc()) {
        static::$variables[$row['name']] = $this->detailsFromId($row['name']);
      }
    }

    // Return the build $variables static array.
    return static::$variables;
  }

  /**
   * Implements RestoreOperation->generate().
   *
   * Now this method generates the restore script action data when
   * the script gets generated, this array should contain any information
   * that is needed to be able to 'restore' it, even if its on another machine
   * or the data does not exist at all when being restored.
   *
   * This may need to also store additional information on dependencies, for
   * example with blocks, they may be a custom block so the custom block
   * information may also need to be stored, or field instances may need
   * the global field details to be able to recreate the field if it doesnt
   * exist.
   *
   * The $id is provided from the generate form, which in turn is provided
   * by the available method.
   */
  public function generate($id) {
    // Variables are very simple, we just need to store the variable name
    // and the variable value.
    return array(
      'name' => $id,
      'value' => variable_get($id),
    );
  }

  /**
   * Implements RestoreOperation->reverse().
   *
   * The reverse method is used when a script is being altered, we convert
   * the data stored in the script in to the data usable for the generate
   * form. The data must be similar to what you would get back from the
   * available method.
   */
  public function reverse(array $action) {
    // Since variables are simple there is not much that needs to be done.
    return array(
      // The ID should be the variable name (see the available method).
      'id' => $action['name'],
      // And the details or label (see the available method).
      'details' => $this->detailsFromId($action['name']),
    );
  }
}
