<?php
/**
 * @file
 * Operation plugin for fields instance restore operations.
 */

$plugin = array(
  'label' => t('Fields instance'),
  'handler' => array(
    'class' => 'RestoreOperationsFieldsInstanceOperation',
  ),
);

class RestoreOperationsFieldsInstanceOperation extends RestoreOperation {
  /** @var array Static storage for field instances for generate. */
  protected static $fieldInstances;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    return t("Field instance '<em>@name</em>' from <em>@entity:@bundle</em>", array(
      '@name' => $action['field_name'],
      '@entity' => $action['entity_type'],
      '@bundle' => $action['bundle'],
    ));
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    $matches = TRUE;

    $instance_info = field_info_instance($action['entity_type'], $action['field_name'], $action['bundle']);
    if (!$instance_info) {
      $matches = FALSE;
    }
    else {
      $keys = array(
        'label',
        'settings',
        'required',
        'display',
        'widget',
        'description',
        'field_name',
        'entity_type',
        'bundle',
        'default_value',
      );

      foreach ($keys as $key) {
        $cval = isset($action[$key]) ? $action[$key] : NULL;
        $ival = isset($instance_info[$key]) ? $instance_info[$key] : NULL;
        if ($cval != $ival) {
          $matches = FALSE;
          break;
        }
      }
    }

    return $instance_info ? ($matches ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_OVERRIDEN) : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    // @TODO Field instance restore.
    $field_info = field_info_field($action['field_name']);
    if (!$field_info) {
      $field = $action['__instance'];
      field_create_field($field);
    }

    $instance_info = field_info_instance($action['entity_type'], $action['field_name'], $action['bundle']);
    try {
      $instance = $action;
      unset($instance['__instance']);

      if ($instance_info) {
        field_update_instance($instance);
      }
      else {
        field_create_instance($instance);
      }

      return TRUE;
    }
    catch (Exception $e) {
      drupal_set_message($e->getMessage(), 'error');
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    // @TODO Field instance remove.
    $instance_info = field_info_instance($action['entity_type'], $action['field_name'], $action['bundle']);
    if ($instance_info) {
      field_delete_instance($action);
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if (!(
        $_action['field_name'] !== $action['field_name'] &&
        $_action['entity_type'] !== $action['entity_type'] &&
        $_action['bundle'] !== $action['bundle']
      )) {
        continue;
      }

      $results[$action_id] = t("Field instance '<em>@name</em>' from <em>@entity:@bundle</em>", array(
        '@name' => $_action['field_name'],
        '@entity' => $_action['entity_type'],
        '@bundle' => $_action['bundle'],
      ));
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    $instance_info = field_info_instance($action['entity_type'], $action['field_name'], $action['bundle']);
    $instance = array();
    $matches = TRUE;

    if ($instance_info) {
      $keys = array(
        'label',
        'settings',
        'required',
        'display',
        'widget',
        'description',
        'field_name',
        'entity_type',
        'bundle',
        'default_value',
      );

      foreach ($keys as $key) {
        $cval = isset($action[$key]) ? $action[$key] : NULL;
        $ival = isset($instance_info[$key]) ? $instance_info[$key] : NULL;
        if ($cval != $ival) {
          $matches = FALSE;
        }

        $instance[$key] = $ival;
      }
    }

    return array(
      'current_value' => $instance_info ? $instance : NULL,
      'restore_value' => $action,
      'match' => $matches,
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    list($entity_type, $bundle_name, $field_name) = explode(RESTORE_GENERATE_ID_SEP, $id);
    $info = field_info_instance($entity_type, $field_name, $bundle_name);

    if ($info) {
      return t("Field instance '<em>@name</em>' from <em>@entity:@bundle</em>", array(
        '@name' => $field_name,
        '@entity' => $entity_type,
        '@bundle' => $bundle_name,
      ));
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$fieldInstances)) {
      $instances = field_info_instances();
      foreach ($instances as $entity_type => $bundles) {
        foreach ($bundles as $bundle_name => $fields) {
          foreach ($fields as $field_name => $info) {
            $id = implode(RESTORE_GENERATE_ID_SEP, array(
              $entity_type,
              $bundle_name,
              $field_name,
            ));

            static::$fieldInstances[$id] = t("Field instance '<em>@name</em>' from <em>@entity:@bundle</em>", array(
              '@name' => $field_name,
              '@entity' => $entity_type,
              '@bundle' => $bundle_name,
            ));
          }
        }
      }
    }

    return static::$fieldInstances;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    list($entity_type, $bundle_name, $field_name) = explode(RESTORE_GENERATE_ID_SEP, $id);
    $keys = array(
      'label',
      'settings',
      'required',
      'display',
      'widget',
      'description',
      'field_name',
      'entity_type',
      'bundle',
      'default_value',
    );

    $instance = field_info_instance($entity_type, $field_name, $bundle_name);
    if (!$instance) {
      return FALSE;
    }

    $field_instance = array();
    foreach ($keys as $key) {
      $field_instance[$key] = isset($instance[$key]) ? $instance[$key] : NULL;
    }

    $field_info = field_info_field($field_name);
    if ($field_info) {
      $field_instance['__instance'] = array();
      $keys = array(
        'translatable',
        'settings',
        'field_name',
        'type',
        'module',
        'active',
        'locked',
        'cardinality',
      );

      foreach ($keys as $key) {
        $field_instance['__instance'][$key] = isset($field_info[$key]) ? $field_info[$key] : NULL;
      }
    }

    return $field_instance;
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    $id = implode(RESTORE_GENERATE_ID_SEP, array(
      $action['entity_type'],
      $action['bundle'],
      $action['field_name'],
    ));

    $details = $this->detailsFromId($id);
    if ($details == FALSE) {
      return FALSE;
    }

    return array(
      'id' => $id,
      'details' => $details,
    );
  }
}
