<?php
/**
 * @file
 * Operation plugin for fields restore operations.
 */

$plugin = array(
  'label' => t('Fields'),
  'handler' => array(
    'class' => 'RestoreOperationsFieldsOperation',
  ),
);

class RestoreOperationsFieldsOperation extends RestoreOperation {
  /** @var array Static storage for the fields for generate. */
  protected static $fields;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    $field_entities = array();
    foreach ($action['instances'] as $instance) {
      $field_entities[$instance['entity_type']][] = $instance['bundle'];
    }

    $field_instances = array();
    foreach ($field_entities as $entity_type => $bundle_names) {
      $field_instances[]  = t('@entity (@bundles)', array(
        '@entity' => $entity_type,
        '@bundles' => implode(', ', $bundle_names),
      ));
    }

    return t("Field '<em>@name</em>' of type <em>@type</em><br /><strong>Available on: </strong><em>@entities</em>", array(
      '@name' => $action['field']['field_name'],
      '@type' => $action['field']['type'],
      '@entities' => implode(', ', $field_instances),
    ));
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    $field_info = field_info_field($action['field']['field_name']);
    $field_matches = $instance_matches = TRUE;

    if ($field_info) {
      $keys = array(
        'translatable',
        'settings',
        'field_name',
        'type',
        'module',
        'active',
        'locked',
        'cardinality',
      );

      foreach ($keys as $key) {
        $cval = isset($action['field'][$key]) ? $action['field'][$key] : NULL;
        $ival = isset($field_info[$key]) ? $field_info[$key] : NULL;
        if ($cval != $ival) {
          $field_matches = FALSE;
          break;
        }
      }

      foreach ($action['instances'] as $instance) {
        $instance_info = field_info_instance($instance['entity_type'], $action['field']['field_name'], $instance['bundle']);
        if (!$instance_info) {
          $instance_matches = FALSE;
        }
        else {
          $keys = array(
            'label',
            'settings',
            'required',
            'display',
            'widget',
            'description',
            'field_name',
            'entity_type',
            'bundle',
            'default_value',
          );

          foreach ($keys as $key) {
            $cval = isset($instance[$key]) ? $instance[$key] : NULL;
            $ival = isset($instance_info[$key]) ? $instance_info[$key] : NULL;
            if ($cval != $ival) {
              $instance_matches = FALSE;
              break;
            }
          }
        }
      }
    }

    return $field_info ? ($field_matches && $instance_matches ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_OVERRIDEN) : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    // @TODO Field restore.
    $field_info = field_info_field($action['field']['field_name']);
    try {
      if ($field_info) {
        field_update_field($action['field']);
      }
      else {
        field_create_field($action['field']);
      }

      if (count($action['instances'])) {
        foreach ($action['instances'] as $instance) {
          $instance_info = field_info_instance($instance['entity_type'], $instance['field_name'], $instance['bundle']);
          if ($instance_info) {
            field_update_instance($instance);
          }
          else {
            field_create_instance($instance);
          }
        }
      }

      return TRUE;
    }
    catch (Exception $e) {
      drupal_set_message($e->getMessage(), 'error');
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    // @TODO Field remove.
    $field_info = field_info_field($action['field']['field_name']);

    if ($field_info) {
      if (count($action['instances'])) {
        foreach ($action['instances'] as $instance) {
          $instance_info = field_info_instance($instance['entity_type'], $instance['field_name'], $instance['bundle']);
          if ($instance_info) {
            field_delete_instance($instance);
          }
        }
      }

      // @TODO Shouldn't really need to do this, as deleting all the instances
      // should delete the field itself too.
      field_delete_field($action['field']['field_name']);
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if (!(
        $_action['field']['field_name'] !== $action['field']['field_name'] &&
        $_action['field']['type'] !== $action['field']['type']
      )) {
        continue;
      }

      $results[$action_id] = t("Field '<em>@name</em>' of type <em>@type</em>", array(
        '@name' => $_action['field']['field_name'],
        '@type' => $_action['field']['type'],
      ));
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    $field_info = field_info_field($action['field']['field_name']);
    $field = array();

    $matches = TRUE;

    if ($field_info) {
      $keys = array(
        'translatable',
        'settings',
        'field_name',
        'type',
        'module',
        'active',
        'locked',
        'cardinality',
      );

      foreach ($keys as $key) {
        $cval = isset($action['field'][$key]) ? $action['field'][$key] : NULL;
        $ival = isset($field_info[$key]) ? $field_info[$key] : NULL;
        if ($cval != $ival) {
          $matches = FALSE;
        }

        $field[$key] = $ival;
      }

      foreach ($action['instances'] as $instance) {
        $instance_info = field_info_instance($instance['entity_type'], $action['field']['field_name'], $instance['bundle']);
        if (!$instance_info) {
          continue;
        }

        $instance_details = array();
        $keys = array(
          'label',
          'settings',
          'required',
          'display',
          'widget',
          'description',
          'field_name',
          'entity_type',
          'bundle',
          'default_value',
        );

        foreach ($keys as $key) {
          $cval = isset($instance[$key]) ? $instance[$key] : NULL;
          $ival = isset($instance_info[$key]) ? $instance_info[$key] : NULL;
          if ($cval != $ival) {
            $matches = FALSE;
          }

          $instance_details[$key] = $ival;
        }
        $field['instances'][] = $instance_details;
      }
    }

    $display = $action['field'];
    $display['instances'] = $action['instances'];

    return array(
      'current_value' => $field_info ? $field : NULL,
      'restore_value' => $display,
      'match' => $matches,
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    $info = field_info_field($id);
    if ($info) {
      $instances = field_info_instances();
      $field_instances = array();

      foreach ($instances as $entity_type => $bundles) {
        foreach ($bundles as $bundle_name => $bundle_instances) {
          if (!isset($bundle_instances[$id])) {
            continue;
          }

          $field_instances[]  = t('@entity (@bundle)', array(
            '@entity' => $entity_type,
            '@bundle' => $bundle_name,
          ));
        }
      }

      return t("Field '<em>@name</em>' of type <em>@type</em><br /><strong>Available on: </strong><em>@entities</em>", array(
        '@name' => $id,
        '@type' => $info['type'],
        '@entities' => implode(', ', $field_instances),
      ));
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$fields)) {
      $fields = field_info_fields();

      foreach ($fields as $field_name => $info) {
        $field_instances = array();

        foreach ($info['bundles'] as $entity_type => $bundle_names) {
          $field_instances[]  = t('@entity (@bundles)', array(
            '@entity' => $entity_type,
            '@bundles' => implode(', ', $bundle_names),
          ));
        }

        static::$fields[$field_name] = t("Field '<em>@name</em>' of type <em>@type</em><br /><strong>Available on: </strong><em>@entities</em>", array(
          '@name' => $field_name,
          '@type' => $info['type'],
          '@entities' => implode(', ', $field_instances),
        ));
      }
    }

    return static::$fields;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    $field = array(
      'field' => array(),
      'instances' => array(),
    );

    $field_info = field_info_field($id);
    if (!$field_info) {
      return FALSE;
    }

    $keys = array(
      'translatable',
      'settings',
      'field_name',
      'type',
      'module',
      'active',
      'locked',
      'cardinality',
    );

    foreach ($keys as $key) {
      $field['field'][$key] = isset($field_info[$key]) ? $field_info[$key] : NULL;
    }

    foreach ($field_info['bundles'] as $entity_type => $bundle_names) {
      foreach ($bundle_names as $bundle_name) {
        $keys = array(
          'label',
          'settings',
          'required',
          'display',
          'widget',
          'description',
          'field_name',
          'entity_type',
          'bundle',
          'default_value',
        );

        $instance = field_info_instance($entity_type, $id, $bundle_name);
        $field_instance = array();
        foreach ($keys as $key) {
          $field_instance[$key] = isset($instance[$key]) ? $instance[$key] : NULL;
        }

        $field['instances'][] = $field_instance;
      }
    }

    return $field;
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    $details = $this->detailsFromId($action['field']['field_name']);
    if ($details == FALSE) {
      return FALSE;
    }

    return array(
      'id' => $action['field']['field_name'],
      'details' => $details,
    );
  }
}
