<?php
/**
 * @file
 * Operation plugin for image style restore operations.
 */

$plugin = array(
  'label' => t('Image styles'),
  'handler' => array(
    'class' => 'RestoreOperationsImageOperation',
  ),
);

class RestoreOperationsImageOperation extends RestoreOperation {
  /** @var array Static storage for the image styles for generate. */
  protected static $styles;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    return filter_xss($action['name']);
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    $style = image_style_load($action['name']);

    $matches = FALSE;
    if ($style && isset($style['isid'])) {
      if (count($style['effects']) == count($action['effects'])) {
        $matches = TRUE;

        $ieids = array_keys($style['effects']);
        foreach ($action['effects'] as $idx => $effect) {
          if (
            ($effect['name'] != $style['effects'][$ieids[$idx]]['name']) ||
            ($effect['data'] != $style['effects'][$ieids[$idx]]['data'])
          ) {
            $matches = FALSE;
            break;
          }
        }
      }
    }

    return $style && isset($style['isid']) ? ($matches ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_OVERRIDEN) : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    $style = image_style_load($action['name']);
    if (!$style || !isset($style['isid'])) {
      $style = array(
        'name' => $action['name'],
        'effects' => array(),
      );

      image_style_save($style);

      $style = image_style_load($action['name']);
    }

    $effects = $style['effects'];

    foreach ($action['effects'] as $action_effect) {
      $effect = array(
        'name' => $action_effect['name'],
      );

      foreach ($effects as $idx => $style_effect) {
        if ($style_effect['name'] == $action_effect['name']) {
          $effect = $style_effect;
          unset($effects[$idx]);
          break;
        }
      }

      $effect['isid'] = $style['isid'];
      $effect['data'] = $action_effect['data'];
      image_effect_save($effect);
    }

    if (count($effects)) {
      foreach ($effects as $effect) {
        image_effect_delete($effect);
      }
    }

    return TRUE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    $style = image_style_load($action['name']);
    if ($style && isset($style['isid'])) {
      image_style_delete($style);
    }

    return TRUE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if (!($_action['name'] !== $action['name'])) {
        continue;
      }

      $results[$action_id] = $this->details($_action);
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    $style = image_style_load($action['name']);

    $matches = FALSE;
    if ($style && isset($style['isid'])) {
      if (count($style['effects']) == count($action['effects'])) {
        $matches = TRUE;

        $ieids = array_keys($style['effects']);
        foreach ($action['effects'] as $idx => $effect) {
          if (
            ($effect['name'] != $style['effects'][$ieids[$idx]]['name']) ||
            ($effect['data'] != $style['effects'][$ieids[$idx]]['data'])
          ) {
            $matches = FALSE;
            break;
          }
        }
      }
    }

    return array(
      'current_value' => $style && isset($style['isid']) ? $style : NULL,
      'restore_value' => $action,
      'match' => $style && isset($style['isid']) && $matches,
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    $style = image_style_load($id);
    if (!$style || !isset($style['isid'])) {
      return FALSE;
    }

    return filter_xss($style['name']);
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$styles)) {
      $styles = image_styles();

      foreach ($styles as $name => $style) {
        if (!isset($style['isid'])) {
          continue;
        }

        static::$styles[$name] = filter_xss($style['name']);
      }
    }

    return static::$styles;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    $style = image_style_load($id);
    if (!$style || !isset($style['isid'])) {
      return FALSE;
    }

    $result = array(
      'name' => $style['name'],
      'effects' => array(),
    );

    foreach ($style['effects'] as $effect) {
      $result['effects'][] = array(
        'name' => $effect['name'],
        'weight' => $effect['weight'],
        'data' => $effect['data'],
      );
    }

    return $result;
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    $style = image_style_load($action['name']);
    if (!$style || !isset($style['isid'])) {
      return FALSE;
    }

    $details = $this->detailsFromId($style['name']);
    if ($details == FALSE) {
      return FALSE;
    }

    return array(
      'id' => $style['name'],
      'details' => $details,
    );
  }
}
