<?php
/**
 * @file
 * Operation plugin for menu restore operations.
 */

$plugin = array(
  'label' => t('Menu and its items'),
  'handler' => array(
    'class' => 'RestoreOperationsMenuOperation',
  ),
);

class RestoreOperationsMenuOperation extends RestoreOperation {
  /** @var array Static storage for the menus for generate. */
  protected static $menus;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    return t("Menu '<em>@name</em>' and its @count link(s).", array(
      '@name' => $action['name'],
      '@count' => isset($action['items']) ? count($action['items']) : 0,
    ));
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    $exists = restore_operations_menu_menu_check($action['name']);
    $matching_items = TRUE;

    if (isset($action['items']) && count($action['items'])) {
      $existing_items = array();
      $dbq = db_select('menu_links', 'ml')
        ->fields('ml')
        ->condition('menu_name', $action['name'])
        ->execute();

      while ($row = $dbq->fetch()) {
        $existing_items[$row->link_path][$row->link_title] = $row;
      }

      if (empty($existing_items)) {
        $matching_items = FALSE;
      }
      else {
        $found = array();
        foreach ($action['items'] as $link) {
          if (!isset($existing_items[$link['path']][$link['title']])) {
            $matching_items = FALSE;
            break;
          }

          $found[$link['path']][$link['title']] = TRUE;
        }

        foreach ($existing_items as $link_path => $link_titles) {
          foreach ($link_titles as $link_title => $link) {
            if (!isset($found[$link_path][$link_title])) {
              $matching_items = FALSE;
              break;
            }
          }
        }
      }
    }

    return $exists ? ($matching_items ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_OVERRIDEN) : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    // Create menu.
    restore_operations_menu_menu_add($action['name'], $action['title'], $action['description']);

    if (restore_operations_menu_menu_check($action['name'])) {
      foreach ($action['items'] as $link) {
        restore_operations_menu_menulink_add($link['title'], $link['path'], $action['name'], $link['weight']);
      }

      return TRUE;
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    // Remove links.
    foreach ($action['items'] as $link) {
      restore_operations_menu_menulink_delete($link['path'], $link['title'], $action['name']);
    }

    // Remove menu.
    $items = db_select('menu_links', 'ml')
      ->fields('ml')
      ->condition('menu_name', $action['name'])
      ->execute()
      ->fetchAll();

    if (empty($items)) {
      restore_operations_menu_menu_delete($action['name']);
    }

    return TRUE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if ($_action['name'] !== $action['name']) {
        continue;
      }

      $results[$action_id] = t("Menu '<em>@name</em>'.", array('@name' => $_action['name']));
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    $exists = restore_operations_menu_menu_check($action['name']);
    $matching_items = TRUE;
    $existing_items_flat = array();
    $menu = menu_load($action['name']);

    if (!$menu) {
      return FALSE;
    }

    if (isset($action['items']) && count($action['items'])) {
      $existing_items = array();
      $dbq = db_select('menu_links', 'ml')
        ->fields('ml')
        ->condition('menu_name', $action['name'])
        ->execute();

      while ($row = $dbq->fetch()) {
        $existing_items[$row->link_path][$row->link_title] = $row;
        $existing_items_flat[] = array(
          'title' => $row->link_title,
          'path' => $row->link_path,
          'weight' => $row->weight,
        );
      }

      if (empty($existing_items)) {
        $matching_items = FALSE;
      }
      else {
        $found = array();
        foreach ($action['items'] as $link) {
          if (!isset($existing_items[$link['path']][$link['title']]) || $existing_items[$link['path']][$link['title']]->weight != $link['weight']) {
            $matching_items = FALSE;
            break;
          }

          $found[$link['path']][$link['title']] = TRUE;
        }

        foreach ($existing_items as $link_path => $link_titles) {
          foreach ($link_titles as $link_title => $link) {
            if (!isset($found[$link_path][$link_title])) {
              $matching_items = FALSE;
              break;
            }
          }
        }
      }
    }

    $value = variable_get($action['name']);
    return array(
      'current_value' => array(
        'name' => $menu['menu_name'],
        'title' => $menu['title'],
        'description' => $menu['description'],
        'items' => $existing_items_flat,
      ),
      'restore_value' => $action,
      'match' => $exists && $matching_items,
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    $count = db_query('SELECT COUNT(*) AS cnt FROM {menu_links} WHERE menu_name = :menu_name', array(':menu_name' => $id))->fetch();
    return t("Menu '<em>@name</em>' and its @count link(s).", array(
      '@name' => $id,
      '@count' => $count->cnt,
    ));
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$menus)) {
      $menus = menu_load_all();

      foreach ($menus as $menu_name => $menu) {
        $count = db_query('SELECT COUNT(*) AS cnt FROM {menu_links} WHERE menu_name = :menu_name', array(':menu_name' => $menu_name))->fetch();
        static::$menus[$menu_name] = t("Menu '<em>@name</em>' and its @count link(s).", array(
          '@name' => $menu_name,
          '@count' => $count->cnt,
        ));
      }
    }

    return static::$menus;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    $menu = menu_load($id);

    $results = array(
      'name' => $id,
      'title' => $menu['title'],
      'description' => $menu['description'],
      'items' => array(),
    );

    $dbq = db_select('menu_links', 'ml', array('fetch' => PDO::FETCH_ASSOC))
      ->fields('ml')
      ->condition('menu_name', $id)
      ->execute();

    while ($row = $dbq->fetch()) {
      $results['items'][] = array(
        'title' => $row['link_title'],
        'path' => $row['link_path'],
        'weight' => $row['weight'],
      );
    }

    return $results;
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    return array(
      'id' => $action['name'],
      'details' => $this->detailsFromId($action['name']),
    );
  }
}
