<?php
/**
 * @file
 * Operation plugin for redirect restore operations.
 */

$plugin = array(
  'label' => t('Redirects'),
  'handler' => array(
    'class' => 'RestoreOperationsRedirectOperation',
  ),
);

class RestoreOperationsRedirectOperation extends RestoreOperation {
  /** @var array Static storage for the redirects for generate. */
  protected static $redirects;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    return t("Redirect '<em>@redirect</em>' => '<em>@source</em>'.", array(
      '@redirect' => $action['redirect'],
      '@source' => $action['source'],
    ));
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    $redirect = restore_operations_redirect_get($action['source']);
    return $redirect ? ((
      $action['source'] == $redirect->source &&
      $action['redirect'] == $redirect->redirect &&
      (isset($action['status_code']) ? $action['status_code'] : 0) == $redirect->status_code &&
      (isset($action['language']) ? $action['language'] : LANGUAGE_NONE) == $redirect->language
    ) ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_OVERRIDEN) : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    if (restore_operations_redirect_check($action['source'])) {
      restore_redirect_delete_redirect($action['source']);
    }

    restore_operations_redirect_add(
      $action['source'],
      $action['redirect'],
      isset($action['status_code']) ? $action['status_code'] : 0,
      isset($action['language']) ? $action['language'] : LANGUAGE_NONE
    );

    return TRUE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    restore_operations_redirect_delete($action['source']);
    return TRUE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if (!(
        $_action['source'] !== $action['source'] &&
        $_action['redirect'] !== $action['redirect']
      )) {
        continue;
      }

      $results[$action_id] = t("Redirect '<em>@redirect</em>' => '<em>@source</em>'.", array(
        '@redirect' => $_action['redirect'],
        '@source' => $_action['source'],
      ));
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    $redirect = restore_operations_redirect_get($action['source']);
    if (!$redirect) {
      return FALSE;
    }

    return array(
      'current_value' => array(
        'source' => $redirect->source,
        'redirect' => $redirect->redirect,
        'status_code' => $redirect->status_code,
        'language' => $redirect->language,
      ),
      'restore_value' => array(
        'source' => $action['source'],
        'redirect' => $action['redirect'],
        'status_code' => isset($action['status_code']) ? $action['status_code'] : '0',
        'language' => isset($action['language']) ? $action['language'] : LANGUAGE_NONE,
      ),
      'match' => (
        $action['source'] == $redirect->source &&
        $action['redirect'] == $redirect->redirect &&
        (isset($action['status_code']) ? $action['status_code'] : 0) == $redirect->status_code &&
        (isset($action['language']) ? $action['language'] : LANGUAGE_NONE) == $redirect->language
      ),
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    $redirect = restore_operations_redirect_get($id);

    if ($redirect) {
      return t("Redirect '<em>@source</em>' => '<em>@redirect</em> (@status_code) [@lang]'.", array(
        '@source' => $redirect->source,
        '@redirect' => $redirect->redirect,
        '@status_code' => $redirect->status_code,
        '@lang' => $redirect->language,
      ));
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$redirects)) {
      $dbq = db_select('redirect', 'rd')
        ->fields('rd', array('source', 'redirect', 'language', 'status_code'))
        ->execute();

      while ($row = $dbq->fetchAssoc()) {
        static::$redirects[$row['source']] = t("Redirect '<em>@source</em>' => '<em>@redirect</em> (@status_code) [@lang]'.", array(
          '@source' => $row['source'],
          '@redirect' => $row['redirect'],
          '@status_code' => $row['status_code'],
          '@lang' => $row['language'],
        ));
      }
    }

    return static::$redirects;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    $redirect = restore_operations_redirect_get($id);
    if ($redirect) {
      return array(
        'source' => $redirect->source,
        'redirect' => $redirect->redirect,
        'status_code' => $redirect->status_code,
        'language' => $redirect->language,
      );
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    $details = $this->detailsFromId($action['source']);
    if ($details == FALSE) {
      return FALSE;
    }

    return array(
      'id' => $action['source'],
      'details' => $details,
    );
  }
}
