<?php
/**
 * @file
 * Operation plugin for rules restore operations.
 */

$plugin = array(
  'label' => t('Rules'),
  'handler' => array(
    'class' => 'RestoreOperationsRulesOperation',
  ),
);

class RestoreOperationsRulesOperation extends RestoreOperation {
  /** @var array Static storage for the rules for generate. */
  protected static $rules;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    return t("Rule '<em>@label</em>'.", array('@label' => $action['label']));
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    $rule = rules_config_load($action['name']);
    $restore_value = rules_import($action['code'], $error_msg);

    $match = ($rule && (
      $rule->label == $restore_value->label &&
      $rule->weight == $restore_value->weight &&
      $rule->name == $restore_value->name &&
      $rule->settings == $restore_value->settings &&
      (!isset($restore_value->active) || $rule->active == $restore_value->active)
    ));

    return $rule ? ($match ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_OVERRIDEN) : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    if ($rules_config = rules_import($action['code'], $error_msg)) {
      try {
        $rules_config->integrityCheck();
      }
      catch (RulesIntegrityException $e) {
        return FALSE;
      }
    }
    else {
      return FALSE;
    }

    $rules_config->name = $action['name'];

    if ($existing_config = rules_config_load($action['name'])) {
      $rules_config->id = $existing_config->id;
      unset($rules_config->is_new);
    }

    $rules_config->save(NULL);

    return TRUE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    $config = rules_config_load($action['name']);
    if (isset($config->id)) {
      rules_config_delete(array($config->id));
    }
    else {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if ($_action['name'] !== $action['name']) {
        continue;
      }

      $results[$action_id] = t("Rule '<em>@name</em>'.", array('@name' => $action['name']));
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    $rule = rules_config_load($action['name']);
    $restore_value = rules_import($action['code'], $error_msg);
    $restore_value->name = $action['name'];

    return array(
      'current_value' => $rule ? $rule : NULL,
      'restore_value' => $restore_value,
      'match' => $rule && (
        $rule->label == $restore_value->label &&
        $rule->weight == $restore_value->weight &&
        $rule->name == $restore_value->name &&
        $rule->settings == $restore_value->settings &&
        (!isset($restore_value->active) || $rule->active == $restore_value->active)
      ),
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    $config = rules_config_load($id);
    if (!$config) {
      return FALSE;
    }

    return t("Rule '<em>@label</em>'.", array('@label' => $config->label));
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$rules)) {
      $configs = rules_config_load_multiple(FALSE);

      foreach ($configs as $config) {
        static::$rules[$config->name] = t("Rule '<em>@label</em>'.", array('@label' => $config->label));
      }
    }

    return static::$rules;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    $config = rules_config_load($id);
    if (!$config) {
      return FALSE;
    }

    return array(
      'name' => $config->name,
      'label' => $config->label,
      'code' => $config->export(),
    );
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    return array(
      'id' => $action['name'],
      'details' => $this->detailsFromId($action['name']),
    );
  }
}
