<?php
/**
 * @file
 * Operation plugin for date format restore operations.
 */

$plugin = array(
  'label' => t('Date format'),
  'handler' => array(
    'class' => 'RestoreDateFormatOperation',
  ),
);

class RestoreDateFormatOperation extends RestoreOperation {
  /** @var array Static storage for the date formats for generate. */
  protected static $formats;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    return t("Date format '<em>@title</em>', @format.", array(
      '@title' => $action['title'],
      '@format' => $action['format'],
    ));
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    $format_types = system_get_date_types();

    return isset($format_types[$action['type']]) ? (variable_get('date_format_' . $action['type']) == $action['format'] ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_OVERRIDEN) : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    $format_types = system_get_date_types();
    $type = isset($format_types[$action['type']]) ? $format_types[$action['type']] : array(
      'type' => $action['type'],
      'title' => $action['title'],
      'locked' => FALSE,
      'is_new' => TRUE,
    );

    system_date_format_type_save($type);

    $date_formats = system_get_date_formats('custom');
    $format = isset($date_formats[$action['format']]) ? $date_formats[$action['format']] : array(
      'type' => 'custom',
      'format' => $action['format'],
      'locked' => FALSE,
    );

    system_date_format_save($format, isset($format['dfid']) ? $format['dfid'] : 0);

    variable_set('date_format_' . $action['type'], $action['format']);

    return TRUE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    $format_types = system_get_date_types();

    if (isset($format_types[$action['type']])) {
      system_date_format_type_delete($action['type']);
      variable_del('date_format_' . $action['type']);

      return TRUE;
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if ($_action['type'] !== $action['type']) {
        continue;
      }

      $results[$action_id] = $this->details($_action);
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    $format_types = system_get_date_types();
    $value = variable_get('date_format_' . $action['type']);

    return array(
      'current_value' => $value,
      'restore_value' => $action['format'],
      'match' => isset($format_types[$action['type']]) && $value == $action['format'],
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    $format_types = system_get_date_types();
    if (!isset($format_types[$id])) {
      return FALSE;
    }

    return t("Date format '<em>@title</em>', @format.", array(
      '@title' => $format_types[$id]['title'],
      '@format' => variable_get('date_format_' . $id),
    ));
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$formats)) {
      drupal_static_reset('system_get_date_types');
      $format_types = system_get_date_types();

      foreach ($format_types as $type => $info) {
        if ($info['locked']) {
          continue;
        }

        static::$formats[$type] = $this->detailsFromId($type);
      }
    }

    return static::$formats;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    $format_types = system_get_date_types();
    if (!isset($format_types[$id])) {
      return FALSE;
    }

    $date_formats = system_get_date_formats('custom');

    return array(
      'title' => $format_types[$id]['title'],
      'type' => $format_types[$id]['type'],
      'format' => variable_get('date_format_' . $id),
    );
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    $format_types = system_get_date_types();
    if (!isset($format_types[$action['type']])) {
      return FALSE;
    }

    return array(
      'id' => $action['type'],
      'details' => $this->detailsFromId($action['type']),
    );
  }
}
