<?php
/**
 * @file
 * Operation plugin for permission restore operations.
 */

$plugin = array(
  'label' => t('Permissions'),
  'handler' => array(
    'class' => 'RestorePermissionOperation',
  ),
);

class RestorePermissionOperation extends RestoreOperation {
  /** @var array Static storage for the permissions during generate. */
  protected static $permissions;

  /**
   * Implements RestoreOperation->details().
   */
  public function details(array $action) {
    return t("Permission '<em>@permission</em>' for role '<em>@role</em>'.", array(
      '@permission' => $action['permission'],
      '@role' => $action['role'],
    ));
  }

  /**
   * Implements RestoreOperation->status().
   */
  public function status(array $action) {
    $role = user_role_load_by_name($action['role']);
    $permissions = $role ? user_role_permissions(array($role->rid => $role->rid)) : array();

    return $role && isset($permissions[$role->rid][$action['permission']]) ? RESTORE_STATUS_INSTALLED : RESTORE_STATUS_UNINSTALLED;
  }

  /**
   * Implements RestoreOperation->restore().
   */
  public function restore(array $action) {
    $role = user_role_load_by_name($action['role']);
    if ($role) {
      user_role_grant_permissions($role->rid, array($action['permission']));
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->remove().
   */
  public function remove(array $action) {
    $role = user_role_load_by_name($action['role']);
    if ($role) {
      user_role_revoke_permissions($role->rid, array($action['permission']));
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Implements RestoreOperation->conflict().
   */
  public function conflict(array $action, array $actions) {
    $results = array();

    foreach ($actions as $action_id => $_action) {
      if (!(
        $_action['permission'] !== $action['permission'] &&
        $_action['role'] !== $action['role']
      )) {
        continue;
      }

      $results[$action_id] = t("Permission '<em>@permission</em>' for role '<em>@role</em>'.", array(
        '@permission' => $_action['permission'],
        '@role' => $_action['role'],
      ));
    }

    return count($results) ? $results : FALSE;
  }

  /**
   * Implements RestoreOperation->diff().
   */
  public function diff(array $action) {
    $role = user_role_load_by_name($action['role']);
    $permissions = $role ? user_role_permissions(array($role->rid => $role->rid)) : array();
    return array(
      'current_value' => $role && isset($permissions[$role->rid][$action['permission']]),
      'restore_value' => TRUE,
      'match' => $role && isset($permissions[$role->rid][$action['permission']]),
    );
  }

  /**
   * Implements RestoreOperation->detailsFromId().
   */
  public function detailsFromId($id) {
    list($role, $permission) = explode(RESTORE_GENERATE_ID_SEP, $id);
    return t("Permission '<em>@permission</em>' for role '<em>@role</em>'.", array(
      '@permission' => str_replace(RESTORE_GENERATE_ID_SUBSEP, ' ', $permission),
      '@role' => str_replace(RESTORE_GENERATE_ID_SUBSEP, ' ', $role),
    ));
  }

  /**
   * Implements RestoreOperation->available().
   */
  public function available() {
    if (empty(static::$permissions)) {
      $roles = user_roles();

      foreach ($roles as $rid => $role) {
        $permissions = user_role_permissions(array($rid => $rid));

        foreach ($permissions as $rid => $perms) {
          foreach ($perms as $permission => $state) {
            $id = implode(RESTORE_GENERATE_ID_SEP, array(
              str_replace(' ', RESTORE_GENERATE_ID_SUBSEP, $role),
              str_replace(' ', RESTORE_GENERATE_ID_SUBSEP, $permission),
            ));

            static::$permissions[$id] = $this->detailsFromId($id);
          }
        }
      }
    }

    return static::$permissions;
  }

  /**
   * Implements RestoreOperation->generate().
   */
  public function generate($id) {
    list($role, $permission) = explode(RESTORE_GENERATE_ID_SEP, $id);

    return array(
      'role' => str_replace(RESTORE_GENERATE_ID_SUBSEP, ' ', $role),
      'permission' => str_replace(RESTORE_GENERATE_ID_SUBSEP, ' ', $permission),
    );
  }

  /**
   * Implements RestoreOperation->reverse().
   */
  public function reverse(array $action) {
    $id = implode(RESTORE_GENERATE_ID_SEP, array(
      str_replace(' ', RESTORE_GENERATE_ID_SUBSEP, $action['role']),
      str_replace(' ', RESTORE_GENERATE_ID_SUBSEP, $action['permission']),
    ));

    return array(
      'id' => $id,
      'details' => $this->detailsFromId($id),
    );
  }
}
