<?php
/**
 * @file
 * Basic class for a restore operation.
 */

abstract class RestoreOperation {
  /**
   * Return the operations details string.
   *
   * This is only used in the output within the forms.
   *
   * @return string
   *   The details string.
   */
  public abstract function details(array $action);

  /**
   * Check the status of the action.
   *
   * Restore statuses:
   * RESTORE_STATUS_UNINSTALLED, RESTORE_STATUS_PARTIAL,
   * RESTORE_STATUS_INSTALLED, RESTORE_STATUS_OVERRIDEN.
   *
   * @return int
   *   Return the status.
   */
  public abstract function status(array $action);

  /**
   * Run the restore operation.
   *
   * @return bool
   *   Returns TRUE or FALSE on success.
   */
  public abstract function restore(array $action);

  /**
   * Run the uninstall/remove operation.
   *
   * @return bool
   *   Returns TRUE or FALSE on success.
   *
   * @access public
   */
  public abstract function remove(array $action);

  /**
   * Check for a conflict.
   *
   * @param array $actions
   *   An array of actions from another script.
   *
   * @return array|bool
   *   If there are any conflicts, an array is returned with the action
   *   index keys, otherwise if no conflicts retunrs FALSE.
   */
  public abstract function conflict(array $action, array $actions);

  /**
   * Compare the restore value to the currently installed value.
   *
   * Compare array:
   * array(
   *   'current_value' => 'The value currently installed.',
   *   'restore_value' => 'The value the restore operation is expecting.',
   *   'match' => TRUE or FALSE (if the values match or not),
   * );
   *
   * @return array
   *   Return the compare array.
   *
   * @access public
   */
  public abstract function diff(array $action);

  /**
   * Return the details of a given action.
   *
   * This is only used in the output within the forms.
   *
   * @return string
   *   The details string.
   */
  public abstract function detailsFromId($id);

  /**
   * Return an array of available values.
   *
   * For example an array of all the variables.
   * Note that the key of the array will be used in the add
   * and remove methods.
   *
   * @example
   *   array(
   *     'id' => 'title',
   *     'id' => 'title',
   *   )
   *
   * @return array
   *   Return an array of available values.
   */
  public abstract function available();

  /**
   * Generate the action output from the given $id.
   *
   * @param string $id
   *   The id of the action to generate.
   *
   * @return bool|array
   *   Returns FALSE if the data is not available/not installed,
   *   otherwise returns the action config array.
   */
  public abstract function generate($id);

  /**
   * This is the opposite of generate.
   *
   * This will take a generated action array and
   * convert it to a $id as expected via generate().
   *
   * @example
   *   $id = 'ID';
   *
   *   $result = $instance->generate($id);
   *   // $result = array('name' => 'ID', 'value' => 'VALUE');
   *
   *   $id = $instance->reverse($result);
   *   // $id = array('id' => 'ID', 'details' => 'Details');
   *
   *   $id = $instance->reverse('doesnt-exist');
   *   // $id = FALSE;
   *
   * @param array $action
   *   The already generated action array.
   *
   * @return bool|array
   *   FALSE if the data is not available/not installed,
   *   otherwise the resulting action id, and details.
   */
  public abstract function reverse(array $action);
}
